@@ -7,16 +7,16 @@
  * Copyright  1999 Anselm Lingnau <lingnau@tm.informatik.uni-frankfurt.de>
  * See file COPYING for conditions on use and distribution.
  *
- * $Id: dvi.h,v 1.4 1999/06/15 16:17:31 lingnau Exp $
+ * $Id: dvi.h,v 1.7 2000/06/29 10:54:42 lingnau Exp $
  */
 
 #ifndef DVI_H
 #define DVI_H
 
 #define DVI_NAME "dvi"
-#define DVI_VERSION "0.1"
+#define DVI_VERSION "0.2"
 #define DVI_MAJOR_VERSION 0
-#define DVI_MINOR_VERSION 1
+#define DVI_MINOR_VERSION 2
 
 #include <stdlib.h>
 #include <time.h>		/* for time_t */
@@ -58,67 +58,6 @@ typedef unsigned long U32;
 #error "We don\'t seem to have a 4-byte data type."
 #endif /* SIZEOF...*/
 
-/*
- * This structure describes a loaded DVI file. It contains various items
- * of information about the file on disk and in memory, and also the
- * usual DVI parameters from the postamble.
- */
-
-typedef struct Dvi_FileInfo {
-    struct Dvi_FileInfo *nextPtr;
-    char *name;			/* Name of the file */
-    dev_t devNo;		/* Device number of the file */
-    ino_t inodeNo;		/* Inode number of the file */
-    int refCount;		/* Reference count for file */
-    int fileDesc;		/* O/S file descriptor */
-    off_t fileSize;		/* Size of file in bytes */
-    time_t lastModTime;		/* Time of last modification of file */
-    U8 *code;			/* Contents of the file */
-    U8 *postamble;		/* Postamble of the file (if available) */
-    
-    U32 num;			/* Numerator for DVI metrics (cf DVI format) */
-    U32 den;			/* Denominator for DVI metrics */
-    U32 mag;			/* Magnification value for document */
-    
-    unsigned int stackSize;	/* Maximum size of DVI stack necessary */
-    unsigned int pageCount;	/* Number of DVI pages in the file */
-    unsigned long generation;	/* Times this file has been reloaded */
-    U8 **pageTable;		/* Table of page offsets */
-} Dvi_FileInfo;
-
-struct Dvi_File;
-typedef void (Dvi_ReloadProc) _ANSI_ARGS_((ClientData clientData,
-					   struct Dvi_File *dviFile));
-
-typedef struct Dvi_File {
-    struct Dvi_File *nextPtr;	/* Next entry in chain */
-    struct Dvi_FileInfo *infoPtr; /* Actual DVI file information */
-    Dvi_ReloadProc *reloadProc; /* Invoked if underlying file was reloaded */
-    ClientData clientData;	/* for reloadProc */
-} Dvi_File;
-
-#define DVI_FILEINFO(d) ((d)->infoPtr)
-
-#define DVI_MIN_STACK 100	/* Minimum DVI stack size (cf. DVI driver */
-				/* standard) (Hallo Joachim!) */
-
-EXTERN Dvi_File *Dvi_OpenFile _ANSI_ARGS_((Tcl_Interp *interp,
-					   const char *name,
-					   Dvi_ReloadProc *reloadProc,
-					   ClientData clientData));
-EXTERN int Dvi_FileChanged _ANSI_ARGS_((Dvi_File *dviFile));
-EXTERN int Dvi_CloseFile _ANSI_ARGS_((Dvi_File *dviFile));
-EXTERN int Dvi_ReloadFile _ANSI_ARGS_((Tcl_Interp *interp, Dvi_File *dviFile));
-
-EXTERN U8 *Dvi_LoadFileBinary _ANSI_ARGS_((const char *name));
-EXTERN Tcl_Obj *Dvi_ListPageNumbers _ANSI_ARGS_((Dvi_File *dviFile));
-
-#ifdef DVI_DEBUG
-EXTERN int Dvi_PurgeFiles _ANSI_ARGS_((void));
-EXTERN int Dvi_FileDump _ANSI_ARGS_((Tcl_DString *string, Dvi_File *filePtr));
-EXTERN int Dvi_FileDumpAll _ANSI_ARGS_((Tcl_DString *string));
-#endif /* DVI_DEBUG */
-
 /*
  * This structure describes a page in terms of its page numbers. A DVI
  * page is identified either by its `absolute' number in the file
@@ -140,15 +79,120 @@ typedef struct Dvi_PageSpec {
 #define DVI_PS_NOSUCHPAGE       (30001)
 #define DVI_PS_NOMATCH          (30002)
 
-EXTERN int Dvi_GetPageSpec _ANSI_ARGS_((Tcl_Interp *interp, char *string,
-                                        Dvi_PageSpec *pageSpecPtr));
+struct Dvi_Code;
 
-EXTERN U8 *Dvi_FindPage _ANSI_ARGS_((Dvi_File *dviFile,
-                                     Dvi_PageSpec *pageSpecPtr,
-                                     unsigned int *currPageNoPtr));
+EXTERN int Dvi_CodeGetPageNumbers _ANSI_ARGS_((struct Dvi_Code *codePtr,
+					       unsigned int absPageNo,
+					       long *pageNoPtr));
+EXTERN int Dvi_CodeFindTeXPage _ANSI_ARGS_((struct Dvi_Code *codePtr,
+					    Dvi_PageSpec *pageSpecPtr));
+EXTERN U8 *Dvi_CodeFindCodeForPage _ANSI_ARGS_((struct Dvi_Code *codePtr,
+						unsigned int absPageNo));
+EXTERN int Dvi_CodeGetPageSpec _ANSI_ARGS_((char *string,
+					    Dvi_PageSpec *pageSpecPtr));
 
-EXTERN U8 *Dvi_FindCodeForAbsolutePage _ANSI_ARGS_((Dvi_File *dviFile,
-						    unsigned int pageNo));
+typedef struct Dvi_PageInfo {
+    U8 *code;			/* Start of DVI code for page */
+} Dvi_PageInfo;
+
+struct Dvi_File;
+struct Dvi_FileInfo;
+
+/*
+ * This data structure describes the structure of a bunch of DVI pages.
+ * This can refer to the contents of a DVI file, but doesn't have to.
+ */
+
+typedef struct Dvi_Code {
+    U32 num;			/* Numerator for DVI metrics (cf DVI format) */
+    U32 den;			/* Denominator for DVI metrics */
+    U32 mag;			/* Magnification value for document */
+    
+    unsigned int stackSize;	/* Maximum size of DVI stack necessary */
+    unsigned int pageCount;	/* Number of DVI pages */
+    
+    unsigned int pageTableSize;	/* Current max size of page table */
+    Dvi_PageInfo *pageTable;
+
+    /* struct Dvi_File *dviFile; */
+} Dvi_Code;
+
+typedef int (Dvi_CodeAnchorRegisterProc) _ANSI_ARGS_((ClientData clientData,
+						      char *anchor,
+						      unsigned int pageNum));
+
+EXTERN Dvi_Code *Dvi_CodeCreate _ANSI_ARGS_((long num, long den, long mag,
+					     unsigned int stackSize,
+					     unsigned int pageCount));
+EXTERN Dvi_Code *Dvi_CodeCreateFromFileInfo
+    _ANSI_ARGS_((struct Dvi_FileInfo*,
+		 Dvi_CodeAnchorRegisterProc *anchorRegisterProc,
+		 ClientData anchorRegisterClientData));
+
+/*
+ * This structure describes a loaded DVI file. It contains various items
+ * of information about the file on disk and in memory.
+ */
+
+typedef struct Dvi_FileInfo {
+    struct Dvi_FileInfo *nextPtr;
+    char *name;			/* Name of the file */
+    dev_t devNo;		/* Device number of the file */
+    ino_t inodeNo;		/* Inode number of the file */
+    int refCount;		/* Reference count for file */
+    int fileDesc;		/* O/S file descriptor */
+    off_t fileSize;		/* Size of file in bytes */
+    time_t lastModTime;		/* Time of last modification of file */
+    U8 *contents;		/* Byte contents of the file */
+    U8 *postamble;		/* Pointer to postamble (if available) */
+    unsigned long generation;	/* Times this file has been reloaded */
+    struct Dvi_Code *codePtr;	/* DVI file structure */
+} Dvi_FileInfo;
+
+typedef void (Dvi_ReloadProc) _ANSI_ARGS_((ClientData clientData,
+					   struct Dvi_File *dviFile));
+typedef int (Dvi_ErrorProc) _ANSI_ARGS_((ClientData clientData,
+					 const char *errMsg));
+typedef struct Dvi_File {
+    struct Dvi_File *nextPtr;	/* Next entry in chain */
+    Dvi_FileInfo *infoPtr;	/* Actual DVI file information */
+    Dvi_ReloadProc *reloadProc; /* Invoked if underlying file was reloaded */
+    ClientData reloadClientData; /* for reloadProc */
+} Dvi_File;
+
+#define DVI_FILEINFO(d) ((d)->infoPtr)
+
+#define DVI_MIN_STACK 100	/* Minimum DVI stack size (cf. DVI driver */
+				/* standard) (Hallo Joachim!) */
+
+EXTERN Dvi_File *Dvi_FileOpen _ANSI_ARGS_((const char *name,
+					   Dvi_ReloadProc *reloadProc,
+					   ClientData reloadClientData,
+					   Dvi_ErrorProc *errorProc,
+					   ClientData errorClientData));
+EXTERN int Dvi_FileChanged _ANSI_ARGS_((Dvi_File *dviFile));
+EXTERN int Dvi_FileClose _ANSI_ARGS_((Dvi_File *dviFile));
+EXTERN int Dvi_FileReload _ANSI_ARGS_((Dvi_File *dviFile,
+				       Dvi_ErrorProc *errorProc,
+				       ClientData errorClientData));
+EXTERN int Dvi_FileParameters _ANSI_ARGS_((Dvi_FileInfo *dviFileInfoPtr,
+					   unsigned char **postPtr,
+					   long *numPtr,
+					   long *denPtr,
+					   long *magPtr,
+					   unsigned int *stackSizePtr,
+					   unsigned int *pageCountPtr));
+EXTERN void Dvi_FileComment _ANSI_ARGS_((Dvi_File *dviFile, char **commentPtr,
+					 unsigned int *lengthPtr));
+
+EXTERN U8 *Dvi_LoadFileBinary _ANSI_ARGS_((const char *name));
+EXTERN Tcl_Obj *Dvi_ListPageNumbers _ANSI_ARGS_((Dvi_File *dviFile));
+
+#ifdef DVI_DEBUG
+EXTERN int Dvi_PurgeFiles _ANSI_ARGS_((void));
+EXTERN int Dvi_FileDump _ANSI_ARGS_((Tcl_DString *string, Dvi_File *filePtr));
+EXTERN int Dvi_FileDumpAll _ANSI_ARGS_((Tcl_DString *string));
+#endif /* DVI_DEBUG */
 
 typedef enum Dvi_FontType {
     dvi_font_pk = 0,
@@ -171,6 +215,7 @@ typedef struct Dvi_Glyph {
     unsigned int bytesWidth;	/* Width in bytes */
     int horizOffset, vertOffset; /* Reference point offset */
     U8 *renderData;		/* For speed on X11 */
+    ClientData fgUid;		/* For colour support. */
     int shrink;			/* Shrink factor for shrinkGlyphPtr */
     int hShrinkOffset, vShrinkOffset;
     void *shrinkGlyphPtr;	/* Cache for shrunken bitmap */
@@ -301,8 +346,8 @@ typedef int (Dvi_FontDefProc) _ANSI_ARGS_((ClientData clientData,
                                            Dvi_FontDefMode mode));
 typedef int (Dvi_SpecialProc) _ANSI_ARGS_((ClientData clientData,
                                            struct Dvi_Interp *dviInterp,
-                                           S32 x, S32 y,
-                                           U32 length, char *string));
+                                           U8 *code, S32 x, S32 y,
+                                           U32 length, const char *string));
 
 /*
  * This structure describes a DVI interpreter.
@@ -310,7 +355,7 @@ typedef int (Dvi_SpecialProc) _ANSI_ARGS_((ClientData clientData,
 
 typedef struct Dvi_Interp {
     Tcl_Interp *interp;		/* Tcl interpreter for error reporting etc. */
-    Dvi_File *dviFile;		/* File that this interpreter works on */
+    Dvi_Code *dviCode;		/* Code that this interpreter works on */
 
     unsigned int xResolution;	/* Horizontal resol. of rendering device */
     unsigned int yResolution;	/* Vertical resolution of rendering device */
@@ -334,10 +379,10 @@ EXTERN Dvi_Interp *Dvi_CreateInterp _ANSI_ARGS_((Tcl_Interp *interp,
                                                  unsigned int yRes,
                                                  unsigned int stackSize,
                                                  U32 num, U32 den, U32 mag));
-EXTERN Dvi_Interp *Dvi_CreateInterpForFile _ANSI_ARGS_((Tcl_Interp *interp,
+EXTERN Dvi_Interp *Dvi_CreateInterpForCode _ANSI_ARGS_((Tcl_Interp *interp,
 							unsigned int xRes,
 							unsigned int yRes,
-							Dvi_File *dviFile));
+							Dvi_Code *dviCode));
 EXTERN int Dvi_SetResolution _ANSI_ARGS_((Dvi_Interp *dviInterp,
                                           unsigned int xRes, unsigned int yRes,
                                           U32 num, U32 den, U32 mag));
@@ -362,19 +407,16 @@ EXTERN Dvi_File *Dvi_GetFileByCookie _ANSI_ARGS_((Tcl_Interp *interp,
 						  const int flags));
 
 /*
- * Procedures from misc.c:
+ * Procedures from dviTcl.c:
  */
 
-EXTERN char * DviSaveStr _ANSI_ARGS_((const char *string));
-EXTERN char * DviSaveStrN _ANSI_ARGS_((const char *string,
-				       const size_t length));
 EXTERN int Dvi_GetPixels _ANSI_ARGS_((Tcl_Interp *interp,
 				      const int resolution, const char *string,
 				      int *result, int flags));
-EXTERN int Dvi_GetDistance _ANSI_ARGS_((Tcl_Interp *interp,
-					const int resolution,
-					const double pixels, const char *unit,
-					double *result, int flags));
+
+EXTERN int Dvi_TclGetPageNumbers _ANSI_ARGS_((Tcl_Interp *interp,
+					      Dvi_Code *codePtr,
+					      Tcl_Obj *pageNoObj));
 
 EXTERN int Dvi_Init _ANSI_ARGS_((Tcl_Interp *interp));
 
