@@ -14,7 +14,7 @@
 #include "dviOps.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: dviInterp.c,v 1.2 1999/06/13 23:22:27 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: dviInterp.c,v 1.5 2000/05/18 12:50:29 lingnau Exp $";
 #endif /* lint */
 
 #define STD_MAXDRIFT 2		/* standard maximum deviation between true */
@@ -30,8 +30,8 @@ static void stdRule _ANSI_ARGS_((ClientData, Dvi_Interp *, S32, S32,
 static int stdFontDef _ANSI_ARGS_((ClientData, Dvi_Interp *, Dvi_FontList **,
 				   S32, U32, U32, U32, size_t, char *,
 				   Dvi_FontDefMode));
-static int stdSpecial _ANSI_ARGS_((ClientData, Dvi_Interp *, S32, S32,
-                                   U32, char *));
+static int stdSpecial _ANSI_ARGS_((ClientData, Dvi_Interp *, U8 *, S32, S32,
+                                   U32, const char *));
 
 /*
  * ------------------------------------------------------------------------
@@ -74,7 +74,7 @@ Dvi_CreateInterp(interp, xRes, yRes, stackSize, numerator, denominator,
      * Allocate memory for the interpreter and stacks.
      */
 
-    dviInterp = ckalloc(sizeof(Dvi_Interp));
+    dviInterp = (Dvi_Interp *)ckalloc(sizeof(Dvi_Interp));
     if (dviInterp == 0) {
         return (Dvi_Interp *)0;
     }
@@ -83,9 +83,10 @@ Dvi_CreateInterp(interp, xRes, yRes, stackSize, numerator, denominator,
      * Allocate the stack with a reserve for virtual fonts.
      */
 
-    dviInterp->stack = ckalloc((stackSize + 10) * sizeof(Dvi_StackFrame));
+    dviInterp->stack = (Dvi_StackFrame *)ckalloc((stackSize + 10)
+						 * sizeof(Dvi_StackFrame));
     if (dviInterp->stack == 0) {
-        ckfree(dviInterp);
+        ckfree((char *)dviInterp);
         return (Dvi_Interp *)0;
     }
     dviInterp->stackSize = stackSize + 10;
@@ -95,6 +96,7 @@ Dvi_CreateInterp(interp, xRes, yRes, stackSize, numerator, denominator,
      */
 
     dviInterp->interp = interp;
+    dviInterp->dviCode = (Dvi_Code *)0;
     Dvi_SetResolution(dviInterp, xRes, yRes, numerator, denominator,
                       magnification);
     dviInterp->maxDrift = STD_MAXDRIFT;
@@ -113,10 +115,10 @@ Dvi_CreateInterp(interp, xRes, yRes, stackSize, numerator, denominator,
 /*
  * ------------------------------------------------------------------------
  *
- * Dvi_CreateInterpForFile --
+ * Dvi_CreateInterpForCode --
  *
  *      Allocates and initializes a Dvi_Interp structure matching a
- *      previously loaded DVI file.
+ *      previously loaded piece of DVI code.
  *
  * Results:
  *      The function returns a pointer to the Dvi_Interp structure or
@@ -129,21 +131,20 @@ Dvi_CreateInterp(interp, xRes, yRes, stackSize, numerator, denominator,
  */
 
 Dvi_Interp *
-Dvi_CreateInterpForFile (interp, xRes, yRes, dviFile)
+Dvi_CreateInterpForCode (interp, xRes, yRes, dviCode)
     Tcl_Interp *interp;		/* Tcl interpreter for error reporting */
     unsigned int xRes;		/* Horizontal resolution of interpreter */
     unsigned int yRes;		/* Vertical resolution of interpreter */
-    Dvi_File *dviFile;		/* DVI file to be used */
+    Dvi_Code *dviCode;		/* DVI file to be used */
 {
     Dvi_Interp *dviInterp;
-    Dvi_FileInfo *dviFileInfo = dviFile->infoPtr;
     
     dviInterp = Dvi_CreateInterp(interp, xRes, yRes,
-				 dviFileInfo->stackSize,
-                                 dviFileInfo->num, dviFileInfo->den,
-				 dviFileInfo->mag);
+				 dviCode->stackSize,
+                                 dviCode->num, dviCode->den,
+				 dviCode->mag);
     if (dviInterp != (Dvi_Interp *)0) {
-        dviInterp->dviFile = dviFile;
+        dviInterp->dviCode = dviCode;
     }
     return dviInterp;
 }
@@ -260,7 +261,7 @@ Dvi_ResetInterp (dviInterp, clearFonts)
 	     listPtr = nextPtr) {
 	    nextPtr = listPtr->nextPtr;
 	    Dvi_FontFree(listPtr->fontPtr);
-	    ckfree(listPtr);
+	    ckfree((char *)listPtr);
 	}
 	dviInterp->fonts = (Dvi_FontList *)0;
     }
@@ -344,7 +345,7 @@ Dvi_ProcessFontDef (dviInterp, fontListPtr, fontNum, fontDefPtr, mode)
     fontScale = DviGetU32(fontDefPtr + 4);
     designSize = DviGetU32(fontDefPtr + 8);
     nameLen = DviGetU8(fontDefPtr+12) + DviGetU8(fontDefPtr+13);
-	
+
     if (mode != DVI_FMODE_DVI) {
 	(dviInterp->fontDefProc)(dviInterp->procData,
 				 dviInterp, fontListPtr, fontNum, check,
@@ -682,19 +683,8 @@ Dvi_DoInterpret(dviInterp, code, f, fontListPtr)
                 
             case D_XXX1: case D_XXX2: case D_XXX3: case D_XXX4:
             {
-                char *specialString;
-                
-                specialString = ckalloc((size_t)(p + 1));
-                if (specialString == 0) {
-                    /*
-                     * Error
-                     */
-                } else {
-		    strncpy(specialString, code, p);
-                    (* dviInterp->specialProc)(dviInterp->procData, dviInterp,
-                                               HH, VV, (U32)p, specialString);
-                    ckfree(specialString);
-                }
+		(* dviInterp->specialProc)(dviInterp->procData, dviInterp,
+					   code, HH, VV, (U32)p, code);
                 code += p;
             }
             
@@ -879,18 +869,19 @@ stdFontDef (clientData, dviInterp, fontListPtr, fontNum,
 }
 
 static int
-stdSpecial (clientData, dviInterp, x, y, length, string)
+stdSpecial (clientData, dviInterp, code, x, y, length, string)
     ClientData clientData;
     Dvi_Interp *dviInterp;
+    U8 *code;
     S32 x, y;
     U32 length;
-    char *string;
+    const char *string;
 {
     Tcl_DString *output = (Tcl_DString *)clientData;
     char buffer[1024];
     
-    sprintf(buffer, "SPECIAL %ld %ld %ld %ld %lu %s\n",
-            (long)HH, (long)VV, (long)x, (long)y,
+    sprintf(buffer, "SPECIAL %ld %ld %p %ld %ld %lu %s\n",
+            (long)HH, (long)VV, code, (long)x, (long)y,
             (unsigned long)length, string);
     if (output != 0) {
         Tcl_DStringAppend(output, buffer, -1);
