@@ -14,7 +14,7 @@
 #include "dviInt.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: font.c,v 1.2 1999/06/13 23:22:27 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: font.c,v 1.4 2000/06/29 11:01:38 lingnau Exp $";
 #endif /* lint */
 
 /*
@@ -116,7 +116,7 @@ Dvi_FontAdd(dviInterp, fontListPtr, fontNum, check, fontScale, designSize,
         return TCL_ERROR;
     }
     
-    newItemPtr = ckalloc(sizeof(Dvi_FontList));
+    newItemPtr = (Dvi_FontList *)ckalloc(sizeof(Dvi_FontList));
     if (newItemPtr == (Dvi_FontList *)0) {
         Dvi_FontFree(fontPtr);
         return TCL_ERROR;
@@ -226,7 +226,7 @@ Dvi_FontNew(dviInterp, check, fontScale, designSize, nameLen, name)
     Dvi_Font *fontPtr;
     U32 fontMag;
 
-    if ((fontPtr = ckalloc(sizeof(Dvi_Font))) == (Dvi_Font *)0) {
+    if ((fontPtr = (Dvi_Font *)ckalloc(sizeof(Dvi_Font))) == (Dvi_Font *)0) {
         return (Dvi_Font *)0;
     }
 
@@ -241,7 +241,7 @@ Dvi_FontNew(dviInterp, check, fontScale, designSize, nameLen, name)
     fontPtr->fileName = Dvi_FindFontFile(nameLen, name, &fontPtr->resolution,
                                          &fontPtr->type);
     if (fontPtr->fileName == (char *)0) {
-        ckfree(fontPtr);
+        ckfree((char *)fontPtr);
         return (Dvi_Font *)0;
     }
 
@@ -258,7 +258,7 @@ Dvi_FontNew(dviInterp, check, fontScale, designSize, nameLen, name)
     if ((* fontTypes[fontPtr->type].loadProc)(dviInterp, fontPtr) != TCL_OK) {
         ckfree(fontPtr->fileName);
         ckfree(fontPtr->fontName);
-        ckfree(fontPtr);
+        ckfree((char *)fontPtr);
         return (Dvi_Font *)0;
     }
 
@@ -394,7 +394,7 @@ GetFontHead (resolution)
 	}
     }
 
-    headPtr = ckalloc(sizeof(FontHead));
+    headPtr = (FontHead *)ckalloc(sizeof(FontHead));
     if (headPtr == (FontHead *)0) {
 	return (FontHead *)0;
     }
@@ -407,15 +407,13 @@ GetFontHead (resolution)
     return headPtr;
 }
 
-#ifdef DVI_DEBUG
 /*
  * ------------------------------------------------------------------------
  *
  * Dvi_FontDump --
  *
  *      Lists information about a font pointed to by `fontPtr', for
- *      debugging purposes. This is only included when debugging is
- *      enabled.
+ *      debugging purposes.
  *
  * Results:
  *      Information about the desired font.
@@ -432,39 +430,33 @@ Dvi_FontDump(interp, fontPtr)
     Dvi_Font *fontPtr;		/* pointer to font to be dumped */
 {
     Tcl_Obj *resultPtr = Tcl_NewListObj(0, (Tcl_Obj **)0);
-    char buf[20];
 
     Tcl_ListObjAppendElement(interp, resultPtr,
 			     Tcl_NewStringObj(fontPtr->fontName, -1));
 
-    sprintf(buf, "%u", fontPtr->resolution);
     Tcl_ListObjAppendElement(interp, resultPtr,
-			     Tcl_NewStringObj(buf, -1));
-
-    sprintf(buf, "%lu", (unsigned long)fontPtr->fontScale);
+			     Tcl_NewIntObj(fontPtr->resolution));
     Tcl_ListObjAppendElement(interp, resultPtr,
-			     Tcl_NewStringObj(buf, -1));
-
+			     Tcl_NewLongObj(fontPtr->fontScale));
+#if 0
     sprintf(buf, "%lu", (unsigned long)fontPtr->designSize);
+#endif
     Tcl_ListObjAppendElement(interp, resultPtr,
-			     Tcl_NewStringObj(buf, -1));
+			     Tcl_NewLongObj(fontPtr->designSize));
 
     Tcl_ListObjAppendElement(interp, resultPtr,
 			     Tcl_NewStringObj(fontTypes[fontPtr->type].name,
 					      -1));
 
-    sprintf(buf, "%d", fontPtr->refCount);
     Tcl_ListObjAppendElement(interp, resultPtr,
-			     Tcl_NewStringObj(buf, -1));
-    sprintf(buf, "%p", fontPtr->fontData);
-    Tcl_ListObjAppendElement(interp, resultPtr,
-			     Tcl_NewStringObj(buf, -1));
+			     Tcl_NewIntObj(fontPtr->refCount));
 
     Tcl_ListObjAppendElement(interp, resultPtr,
 			     Tcl_NewStringObj(fontPtr->fileName, -1));
     return resultPtr;
 }
 
+#ifdef DVI_DEBUG
 /*
  * ------------------------------------------------------------------------
  *
@@ -586,3 +578,4 @@ Dvi_FontDumpGlyph(dviInterp, fontSize, name, character, string)
     return TCL_OK;
 }
 #endif /* DVI_DEBUG */
+
